-- This Source Code Form is subject to the terms of the bCDDL, v. 1.1.
-- This Source Code Form is subject to the terms of the bCDDL, v. 1.1.
-- If a copy of the bCDDL was not distributed with this
-- file, You can obtain one at http://beamng.com/bCDDL-1.1.txt

---------------------
-- Prep the module --
---------------------
local M = {}
M.type = "auxilliary"
M.relevantDevice = nil
M.defaultOrder = 11000

-----------------
-- Debug stuff --
-----------------
local ugfxCalled = 0

---------------------------
-- Tuning menu variables --
---------------------------

-- Duration of timed launch
local launchDuration = 0.2

-- Not a timed launch
-- Continue as long as button is held pressed
local launch4Ever = 0

-- Pressing the button toggles the thruster on/off
-- Press-Release --> ON
-- Press-Release --> OFF
local launchToggle = 0

---------------------
-- State variables --
---------------------

-- Elapsed launch time as of right now
local launchElapsed = 0

-- Last trigger states
local lastTrig = 0
local lastTrigR = 0

-- Mortar Firing Direction (mortarDir)
-- 0 = Neither
-- 1 = Forward
-- 2 = Reverse

-- Detected edges
-- 0 = No edges
-- 1 = Rising edge
-- 2 = Falling edge

-------------------------------
-- Terminating mortar firing --
-------------------------------
local function launchTerminate()
	electrics.values['mortarFiring'] = 0
	electrics.values['mortarFiringR'] = 0
end

------------------------------------
-- Determine edges on triggers    --
------------------------------------
local function getTriggerEdges()
	local	edgeDir = 0
	local	edgeDirR = 0
	
	-- Trigger activated
	local mortarTrigger = (electrics.values['mortarTrigger'] or 0)
	local mortarTriggerR = (electrics.values['mortarTriggerR'] or 0)
	
	-- Detect forward edge
	if mortarTrigger == 1 then	if lastTrig == 0 then edgeDir = 1 end
	else						if lastTrig == 1 then edgeDir = 2 end end
	
	-- Detect reverse edge
	if mortarTriggerR == 1 then	if lastTrigR == 0 then edgeDirR = 1 end
	else						if lastTrigR == 1 then edgeDirR = 2 end end
	
	-- New trigger states have become old trigger states
	lastTrig = mortarTrigger
	lastTrigR = mortarTriggerR
	
	-- Return result
	return edgeDir, edgeDirR
end

---------------------------------------------
-- Graphics update function                --
-- Updates other stuff too according to dt --
---------------------------------------------
local function updateGFX(dt)
	
	-- Which direction are we firing: fwd/rev?
	local edgeDir
	local edgeDirR
	edgeDir, edgeDirR = getTriggerEdges()
	
	-- Retrieve some electrics
	local mortarFiring	= (electrics.values['mortarFiring']		or 0)	-- Forward thruster ON
	local mortarFiringR	= (electrics.values['mortarFiringR']	or 0)	-- Reverse thruster ON
	
	-----------------------
	-- Timed firing mode --
	-----------------------
	if launch4Ever <= 0.5 then
		
		-- Already firing; Keep going until timeout
		if mortarFiring == 1 or mortarFiringR == 1 then
			launchElapsed = launchElapsed + dt
			if launchElapsed >= launchDuration then launchTerminate() end
			
		-- Not firing; Fire on rising edge
		-- Forward has precedence over reverse
		else
			if		edgeDir == 1	then electrics.values['mortarFiring']	= 1
			elseif	edgeDirR == 1	then electrics.values['mortarFiringR']	= 1
			end
			launchElapsed = 0
		end
		
	----------------------------
	-- Continuous firing mode --
	----------------------------
	else
		
		----------------------------
		-- Momentary trigger mode --
		----------------------------
		if launchToggle <= 0.5 then
			
			-- Already firing in forward or reverse direction
			-- Terminate thruster if terminating edge found
			if		mortarFiring == 1	then if edgeDir == 2	then launchTerminate() end
			elseif	mortarFiringR == 1	then if edgeDirR == 2	then launchTerminate() end
				
			-- Not firing in either direction
			-- Fire in whatever direction is commanded
			else
				if		edgeDir == 1	then electrics.values['mortarFiring']	= 1
				elseif	edgeDirR == 1	then electrics.values['mortarFiringR']	= 1 end
			end
			
		--------------------------
		-- Toggled trigger mode --
		--------------------------
		else
			
			-- Already firing in the forward direction
			if mortarFiring == 1 then
			
				-- Rising edge in forward direction; Toggle
				if edgeDir == 1 then
					electrics.values['mortarFiring'] = 0
					
				-- Rising edge in reverse direction; Switch to reverse
				elseif edgeDirR == 1 then
					electrics.values['mortarFiring'] = 0
					electrics.values['mortarFiringR'] = 1
				end
				
			-- Already firing in the reverse direction
			elseif mortarFiringR == 1 then
				
				-- RIsing edge in reverse direction; Toggle
				if edgeDirR == 1 then
					electrics.values['mortarFiringR'] = 0
					
				-- Rising edge in forward direction; Switch to forward
				elseif edgeDir == 1 then
					electrics.values['mortarFiringR'] = 0
					electrics.values['mortarFiring'] = 1
				end
				
			-- Not firing in either direction
			-- Fire in whatever direction commanded
			else
				if		edgeDir == 1 then electrics.values['mortarFiring']	= 1
				elseif	edgeDirR == 1 then electrics.values['mortarFiringR']	= 1 end
			end
		end
	end
	
	-- Debug message
	if ugfxCalled == 0 then
		ugfxCalled = 1
		print("Stunt Mortar updateGFX called")
	end
end

-- Reset what we need to
local function reset()
	launchElapsed = 0
	lastTrig = 0
	lastTrigR = 0
	launchTerminate()
	print("Stunt Mortar reset called")
end

-- Initialize everything
local function init(jbeamdata)
	launchDuration = (v.data.variables["$hb9k_forceDur"].val or 200) / 1000
	launch4Ever = (v.data.variables["$hb9k_force4Ever"].val or 0)
	launchToggle = (v.data.variables["$hb9k_forceToggle"].val or 0)
	print("Stunt Mortar init called")
end

-- Public interface
M.init = init
M.reset = reset
M.updateGFX = updateGFX

-- Return the module
return M
